<?php

namespace Drupal\privatemsg\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\privatemsg\Entity\PrivatemsgMessage;
use Drupal\privatemsg\Entity\PrivatemsgThread;
use Drupal\privatemsg\PrivateMsgService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a Private Messages form.
 */
class PrivatemsgComplainForm extends FormBase {

  /**
   * The entity type manager.
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The current user account.
   */
  protected AccountInterface $currentUser;

  /**
   * Common functions.
   */
  protected PrivateMsgService $privateMsgService;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'privatemsg_complain_form';
  }

  /**
   * The form constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   Current user.
   * @param \Drupal\privatemsg\PrivateMsgService $privatemsg_service
   *   Common functions.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, AccountInterface $current_user, PrivateMsgService $privatemsg_service) {
    $this->entityTypeManager = $entity_type_manager;
    $this->currentUser = $current_user;
    $this->privateMsgService = $privatemsg_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('current_user'),
      $container->get('privatemsg.common'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $thread_id = NULL, $message_id = NULL) {
    $claim_reason_terms = $this->entityTypeManager->getStorage('taxonomy_term')->loadTree('privatemsg_claim_reason');
    $claim_reason_options = [];
    foreach ($claim_reason_terms as $claim_reason_term) {
      $claim_reason_options[$claim_reason_term->tid] = $claim_reason_term->name;
    }

    $form['claim_reason'] = [
      '#type' => 'select',
      '#options' => $claim_reason_options,
      '#title' => $this->t('Claim reason'),
      '#required' => TRUE,
    ];

    $form['message'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Message'),
      '#required' => TRUE,
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Send'),
    ];

    if ($thread_id) {
      $form_state->set('thread_id', $thread_id);
    }
    if ($message_id) {
      $form_state->set('message_id', $message_id);
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $config = $this->configFactory()->get('privatemsg.settings');
    $moderator_role = $config->get('moderator_role');
    if (!$moderator_role) {
      $form_state->setErrorByName('', $this->t("Can not send message to moderator. Moderator role doesn't selected."));
    }
    $users = $this->entityTypeManager->getStorage('user')->getQuery()
      ->condition('status', 1)
      ->condition('roles', $moderator_role)
      ->accessCheck(FALSE)
      ->count()
      ->execute();
    if ($users === 0) {
      $form_state->setErrorByName('', $this->t('Can not send message to moderator. There is no any user with moderator role.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->configFactory()->get('privatemsg.settings');
    $moderator_role = $config->get('moderator_role');
    $user_ids = $this->entityTypeManager->getStorage('user')->getQuery()
      ->condition('status', 1)
      ->condition('roles', $moderator_role)
      ->accessCheck(FALSE)
      ->execute();
    $user_ids[$this->currentUser->id()] = $this->currentUser->id();

    $message_text = '';

    $claim_reason_id = $form_state->getValue('claim_reason');
    $claim_reason_term = $this->entityTypeManager->getStorage('taxonomy_term')->load($claim_reason_id);
    if ($claim_reason_term) {
      $message_text .= $this->t('Reason:') . ' ' . $claim_reason_term->getName() . '<br>';
    }
    $message_from_user = $form_state->getValue('message');
    $message_text .= $this->t('Message from user:') . ' ' . $message_from_user . '<br>';
    $message_id = $form_state->get('message_id');
    if ($message_id) {
      $bad_message = $this->entityTypeManager->getStorage('privatemsg_message')->load($message_id);
      if ($bad_message) {
        $bad_message_text = $bad_message->get('message')->value;
        $message_text .= $this->t('Original text:') . ' ' . $bad_message_text . '<br>';
      }
    }
    $thread_id = $form_state->get('thread_id');
    if ($thread_id) {
      $url = Url::fromRoute('entity.privatemsg_thread.canonical', [
        'privatemsg_thread' => $thread_id,
      ], [
        'absolute' => TRUE,
      ])->toString();
      $message_text .= $this->t('Url to thread:') . " <a href='$url'>" . $url . '</a>';
    }

    $message = PrivatemsgMessage::create([
      'owner' => $this->currentUser->id(),
      'message' => [
        'value' => Markup::create($message_text),
        'format' => 'full_html',
      ],
    ]);
    $message->save();

    $threads_group = $this->privateMsgService->getLastThreadGroupNumber() + 1;

    $system_tag = $this->entityTypeManager->getStorage('taxonomy_term')->loadByProperties([
      'vid' => 'privatemsg_tags',
      'field_privatemsg_tag_m_name' => 'system',
    ]);
    $system_tag = reset($system_tag);

    foreach ($user_ids as $user_id) {
      $thread = PrivatemsgThread::create([
        'subject' => $this->t('Claim to moderator'),
        'owner' => $user_id,
        'private_messages' => $message->id(),
        'members' => $user_ids,
        'group' => $threads_group,
        'tags' => [$system_tag->id()],
      ]);
      $thread->save();
    }
    $this->messenger()->addStatus($this->t('The message has been sent to moderator.'));
  }

}
